#include<qapplication.h>
#include<qtextedit.h>
#include<qgroupbox.h>
#include<qlineedit.h>
#include<qlabel.h>
#include<qcheckbox.h>
#include<qtextedit.h>
#include<qcombobox.h>
#include<qpushbutton.h>
#include<qmessagebox.h>
#include<qinputdialog.h>
#include<qspinbox.h>
#include<qtimer.h>
#include<qmenubar.h>
#include<qpopupmenu.h>
#include<qtabwidget.h>
#include<qca.h>
#include <qsocket.h>
#include <qserversocket.h>
#include <qvbox.h>
#include <qtextview.h>
//#include<iris/xmpp.h>
#include"xmpp.h"
#include"im.h"

#include"xmpp_tasks.h"
#include"xmpp_jingle.h"

#include<stdlib.h>
#include<time.h>

#include<stdio.h>

/*
  The ClientSocket class provides a socket that is connected with a client.
  For every client that connects to the server, the server creates a new
  instance of this class.
*/
class ClientSocket : public QSocket
{
    Q_OBJECT
public:
    ClientSocket( int sock, QObject *parent=0, const char *name=0 ) :
	QSocket( parent, name )
    {
	connect( this, SIGNAL(readyRead()),
		SLOT(readClient()) );
	connect( this, SIGNAL(connectionClosed()),
		SLOT(deleteLater()) );
	setSocket( sock );
    }

    ~ClientSocket()
    {
    }

signals:
    void logText( const QString& );

private slots:
    void readClient()
    {
		QTextStream ts( this );
		QString str;
		while ( canReadLine() )
			str += ts.readLine() + "\n";
	    emit logText( str );
/*
	    ts << line << ": " << str << endl;
	    emit logText( tr("Wrote: '%1: %2'\n").arg(line).arg(str) );*/
    }
};


/*
  The SimpleServer class handles new connections to the server. For every
  client that connects, it creates a new ClientSocket -- that instance is now
  responsible for the communication with that client.
*/
class SimpleServer : public QServerSocket
{
    Q_OBJECT
public:
    SimpleServer( QObject* parent=0 ) :
	QServerSocket( 4242, 1, parent )
    {
	if ( !ok() ) {
	    qWarning("Failed to bind to port 4242");
	    exit(1);
	}
    }

    ~SimpleServer()
    {
    }

    void newConnection( int socket )
    {
	ClientSocket *s = new ClientSocket( socket, this );
	emit newConnect( s );
    }

signals:
    void newConnect( ClientSocket* );
};


/*
  The ServerInfo class provides a small GUI for the server. It also creates the
  SimpleServer and as a result the server.
*/
class ServerInfo : public QVBox
{
    Q_OBJECT
public:
    ServerInfo()// : QVBox(0, 0, WStyle_StaysOnTop)
    {
	SimpleServer *server = new SimpleServer( this );

	QString itext = tr(
		"This is a small server example.\n"
		"Connect with the client now."
		);
	QLabel *lb = new QLabel( itext, this );
	lb->setAlignment( AlignHCenter );
	outText = new QTextView( this );
	inText = new QTextView( this );
	QPushButton *accept = new QPushButton( tr("Accept") , this );
	QPushButton *decline = new QPushButton( tr("Decline") , this );
	QPushButton *redirect = new QPushButton( tr("Redirect") , this );
	QPushButton *terminate = new QPushButton( tr("Terminate") , this );
	QPushButton *quit = new QPushButton( tr("Quit") , this );

	connect( server, SIGNAL(newConnect(ClientSocket*)), SLOT(newConnect(ClientSocket*)) );
	connect( accept, SIGNAL(clicked()), XMPP::JingleSession::session(), SLOT(accept()) );
	connect( decline, SIGNAL(clicked()), XMPP::JingleSession::session(), SLOT(decline()) );
	connect( redirect, SIGNAL(clicked()), SLOT(redirect()) );
	connect( terminate, SIGNAL(clicked()), XMPP::JingleSession::session(), SLOT(terminate()) );
	connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );
	conn = new XMPP::AdvancedConnector;
	stream = new XMPP::ClientStream(conn, 0);
	client = new XMPP::Client(this);
	XMPP::Jid jid("juliet@capulet.com/balcony");
	client->connectToServer(stream, jid);
	client->start(jid.host(), jid.user(), "", jid.resource());
	connect(client, SIGNAL(xmlOutgoing(const QString &)), SLOT(writed(const QString &)));
	socket = 0;
	setGeometry(10, 380, 600, 400);
    }

    ~ServerInfo()
    {
		delete stream;
		delete conn;
    }

private slots:
    void newConnect( ClientSocket *s )
    {
	if (socket) return;
	socket = s;
	connect( s, SIGNAL(logText(const QString&)),
		SLOT(readed(const QString&)) );
	connect( s, SIGNAL(connectionClosed()),
		SLOT(connectionClosed()) );
    }

    void connectionClosed()
    {
	socket = 0;
    }
    
    void readed(const QString &s)
    {
		inText->append(s);
		QDomDocument doc;
		doc.setContent(s);
		QDomElement e = doc.firstChild().toElement();
		client->rootTask()->take(e);
    }
    
    void writed(const QString &s)
    {
		if(!socket) return;
		QDomDocument doc;
		doc.setContent(s);
		QDomElement e = doc.firstChild().toElement();
		e.setAttribute("from", "juliet@capulet.com/balcony");
		QTextStream os(socket);
		os<<doc.toString()<< "\n";
		outText->append(doc.toString());
    }
    
	void redirect()
	{
		XMPP::JingleSession::session()->redirect("juliet@capulet.com/chamber");
	}

private:
    QTextView *inText, *outText;
    XMPP::Client * client;
    ClientSocket *socket;
    XMPP::ClientStream *stream;
    XMPP::AdvancedConnector *conn;
};

#include"xmpptest.moc"

int main(int argc, char **argv)
{
#ifdef Q_OS_WIN32
	QApplication::addLibraryPath(".");
	putenv("SASL_PATH=.\\sasl");
#endif
	QApplication app(argc, argv);

	// seed the random number generator (needed at least for HttpPoll)
	srand(time(NULL));
	ServerInfo info;
    app.setMainWidget( &info );
    info.show();
	QObject::connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
	app.exec();
	// we need this for a clean exit
	QCA::unloadAllPlugins();

	return 0;
}
