#include "xmpp_jingle.h"
#include"xmpp_xmlcommon.h"

using namespace XMPP;

const QString FID_JINGLE = "http://jabber.org/protocol/jingle";

typedef QMap<int, QString> ActionsMap;
ActionsMap actionsMap;
					
enum Action {		
	Accept,		//Required Actions
	Info,
	Initiate,
	Negotiate,
	Redirect,
	Terminate,
	Join,		//Optional Actions
	Modify,
	Replace,
	Transfer
};
					
class JingleManager::Private
{
public:
	Client *client;
	JT_Jingle *jtj;
};

JingleManager::JingleManager(Client *client)
:QObject(client)
{
	d = new Private;
	d->client = client;
	d->jtj = new JT_Jingle(d->client->rootTask(), true);
}

JingleManager::~JingleManager()
{
	delete d;
}

Client *JingleManager::client() const
{
	return d->client;
}

void JT_Jingle::getDiscoInfo(QDomElement *query, QDomDocument *doc)
{
	actionsMap[Accept] = "accept";	//Required Actions
	actionsMap[Info] = "info";
	actionsMap[Initiate] = "initiate";
	actionsMap[Negotiate] = "negotiate";
	actionsMap[Redirect] = "redirect";
	actionsMap[Terminate] = "terminate";
	actionsMap[Join] = "join";		//Optional Actions
	actionsMap[Modify] = "modify";
	actionsMap[Replace] = "replace";
	actionsMap[Transfer] = "transfer";

	QDomElement feature = doc->createElement("feature");
	feature.setAttribute("var", FID_JINGLE );
	query->appendChild(feature);
	
	feature = doc->createElement("feature");
	feature.setAttribute("var", FID_JINGLE + "/sessions/audio");
	query->appendChild(feature);
	
	feature = doc->createElement("feature");
	feature.setAttribute("var", FID_JINGLE + "?mode=dribble");
	query->appendChild(feature);
	
	feature = doc->createElement("feature");
	feature.setAttribute("var", FID_JINGLE + "?mode=burst");
	query->appendChild(feature);
	
	feature = doc->createElement("feature");
	feature.setAttribute("var", FID_JINGLE + "?stun=inline");
	query->appendChild(feature);
	
	for(ActionsMap::Iterator it = actionsMap.begin(); it != actionsMap.end(); ++it) {
		feature = doc->createElement("feature");
		feature.setAttribute("var", FID_JINGLE + "?action=" + it.data());
		query->appendChild(feature);
	}
}

class JT_Jingle::Private
{
public:
	QDomElement iq;
	bool serve;
	Jid to;
	QString id;
};

JT_Jingle::JT_Jingle(Task *parent, bool serve)
	:Task(parent)
{
	d = new Private;
	d->serve = serve;
}

JT_Jingle::~JT_Jingle()
{
	delete d;
}

bool JT_Jingle::take(const QDomElement &e)
{
	if(e.tagName() != "iq")
		return false;

	QString type = e.attribute("type");
	Jid from(e.attribute("from"));
	
	if (d->serve) {
		if (type != "set")
			return false;
		
		QDomElement x = e.firstChild().toElement();
		if(x.tagName() != "jingle")
			return false;
		d->id = e.attribute("id");
#ifdef DEBUG
		d->to = Jid(e.attribute("to"));
#endif
        incomingRequest(x);
	}
	else {
		if(e.attribute("id") != d->id || !d->to.compare(from))
			return false;
		if (type == "result") {
			setSuccess();
		}
		else
			setError(e);
	}
#ifdef DEBUG
	d->serve = !d->serve;
#endif
	return true;
}

void JT_Jingle::onGo()
{
	send(d->iq);
}

void JT_Jingle::incomingRequest(const QDomElement &x)
{
	QString action = x.attribute("action"); 
//    QString initiator = x.attribute("initiator");
//    QString sid = x.attribute("sid");
	int numAction = -1;
	for(ActionsMap::Iterator it = actionsMap.begin(); it != actionsMap.end(); ++it)
		if (action == it.data()) {
			numAction = it.key();
			break;
		}

	switch(numAction) {
		case Accept:
			accept(x);
			break;
		case Info:
			info(x);
			break;
		case Initiate:
			initiate(x);
			break;
		case Negotiate:
			negotiate(x);
			break;
		case Redirect:
			redirect(x);
			break;
		case Terminate:
			terminate(x);
			break;
		case Join:
			join(x);
			break;
		case Modify:
			modify(x);
			break;
		case Replace:
			replace(x);
			break;
		case Transfer:
			transfer(x);
			break;
		default:
			return;
	}
}

void JT_Jingle::accept(const QDomElement &x)
{
}

void JT_Jingle::info(const QDomElement &x)
{
}

void JT_Jingle::initiate(const QDomElement &x)
{
}

void JT_Jingle::negotiate(const QDomElement &x)
{
}

void JT_Jingle::redirect(const QDomElement &x)
{
}

void JT_Jingle::terminate(const QDomElement &x)
{
}

void JT_Jingle::join(const QDomElement &x)
{
}

void JT_Jingle::modify(const QDomElement &x)
{
}

void JT_Jingle::replace(const QDomElement &x)
{
}

void JT_Jingle::transfer(const QDomElement &x)
{
}